@@ -1,9 +1,5 @@
 #include <signal.h>
-#include <stddef.h>
 #include "syscall.h"
-#include "libc.h"
-
-int __sigaction(int, const struct sigaction *, struct sigaction *);
 
 void (*signal(int sig, void (*func)(int)))(int)
 {
@@ -14,3 +10,4 @@ void (*signal(int sig, void (*func)(int)))(int)
 }
 
 weak_alias(signal, bsd_signal);
+weak_alias(signal, __sysv_signal);
